<?php

declare(strict_types=1);

namespace Dedi\SyliusMegamenuPlugin\Form\Listener;

use Symfony\Component\Form\FormEvent;
use Symfony\Component\String\Slugger\SluggerInterface;

final class AdFileUploader implements AdFileUploaderInterface
{
    /** @var string */
    private $publicPath;

    /** @var string*/
    private $imagePath;

    /** @var SluggerInterface */
    private $slugger;

    public function __construct(
        string $publicPath,
        string $imagePath,
        SluggerInterface $slugger
    ) {
        $this->slugger = $slugger;
        $this->publicPath = $publicPath;
        $this->imagePath = $imagePath;
    }

    public function __invoke(FormEvent $event): void
    {
        $fileAware = $event->getForm()->getData();

        if (!$fileAware instanceof AdAwareInterface) {
            throw new \LogicException(sprintf(
                'Expected object of type "%s", got "%s"',
                AdAwareInterface::class,
                get_class($fileAware)
            ));
        }

        if ($event->getForm()->get('removeFile')->getData() === true) {
            $fileAware->setAdImage(null);

            return;
        }


        $file = $event->getForm()->get('file')->getData();
        if (null === $file) {
            return;
        }

        $originalFilename = pathinfo($file->getClientOriginalName(), PATHINFO_FILENAME);
        $safeFilename = $this->slugger->slug($originalFilename);
        $newFilename = $safeFilename.'-'.uniqid().'.'.$file->guessExtension();

        $file->move(sprintf('%s/%s', $this->publicPath, $this->imagePath), $newFilename);
        $fileAware->setAdImage(sprintf('/%s/%s', $this->imagePath, $newFilename));
    }
}
